"use strict";
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) result[k] = mod[k];
    result["default"] = mod;
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
const ts_simple_ast_1 = require("ts-simple-ast");
const ts = __importStar(require("typescript"));
const typescript_ast_util_1 = require("typescript-ast-util");
exports.codeFixCreateConstructor = {
    name: 'Declare constructor',
    config: { variableType: 'const' },
    predicate: (diag, child, log) => {
        if (!diag.find(d => d.code === 2554)) {
            return false;
        }
        if (child.kind === ts.SyntaxKind.NewExpression) {
            return true;
        }
        else {
            log(`codeFixCreateConstructor predicate false because  child.kind didn't match : child.kind==${typescript_ast_util_1.getKindName(child.kind)}, child.parent.kind==${typescript_ast_util_1.getKindName(child.parent.kind)}`);
            return false;
        }
    },
    description: (ds, child) => `Declare constructor "${child.getText()}"`,
    apply: (diag, node, log) => {
        const originalKind = node.getKind();
        if (!ts_simple_ast_1.TypeGuards.isNewExpression(node)) {
            node = node.getFirstAncestorByKind(ts.SyntaxKind.NewExpression);
        }
        if (!node || !ts_simple_ast_1.TypeGuards.isNewExpression(node)) {
            log(`codeFixCreateConstructor apply fail because couldnt find a NewExpression from node returned by sourcefile..getDescendantAtPos(positionOrRangeToNumber(positionOrRange) - returned node kind was ${typescript_ast_util_1.getKindName(originalKind)}`);
            return;
        }
        const argTypes = node.getArguments().map(arg => arg.getType().getApparentType().getText());
        const classDeclaration = node.getExpression().getSymbol().getDeclarations()[0];
        if (ts_simple_ast_1.TypeGuards.isClassDeclaration(classDeclaration)) {
            classDeclaration.addConstructor({
                // docs: 'Autogenerated constructor', //TODO. add some jsdoc to generated constructor
                scope: ts_simple_ast_1.Scope.Public,
                parameters: argTypes.map(type => ({
                    name: `a${type.substring(0, 1).toUpperCase()}${type.substring(1, type.length)}`,
                    hasQuestionToken: false,
                    type,
                    isRestParameter: false,
                }))
            });
        }
        else {
            log(`codeFixCreateConstructor apply fail because node is not ClassDeclaration is ${typescript_ast_util_1.getKindName(classDeclaration.getKind())}`);
        }
    }
};
//# sourceMappingURL=codeFixCreateConstructor.js.map