import { Node, Scope, TypeGuards } from 'ts-simple-ast';
import * as ts from 'typescript';
import { CodeFix } from './codeFixes';
import { getKindName } from 'typescript-ast-util';

export const codeFixCreateConstructor: CodeFix = {
  name: 'Declare constructor',
  config: { variableType: 'const' },
  
  predicate: (diag: ts.Diagnostic[], child: ts.Node, log: (str: string) => void) => {
    if (!diag.find(d=>d.code === 2554)) {
      return false
    }
    if (child.kind === ts.SyntaxKind.NewExpression) {
      return true
    } else {
      log(`codeFixCreateConstructor predicate false because  child.kind didn't match : child.kind==${getKindName(child.kind)}, child.parent.kind==${getKindName(child.parent.kind)}`)
      return false
    }
  },

  description: (ds: ts.Diagnostic[], child: ts.Node): string => `Declare constructor "${child.getText()}"`,

  apply: (diag: ts.Diagnostic[], node: Node, log) => {
    const originalKind = node.getKind()
    if (!TypeGuards.isNewExpression(node)) {
      node = node.getFirstAncestorByKind(ts.SyntaxKind.NewExpression)
    }
    if (!node || !TypeGuards.isNewExpression(node)) {
      log(`codeFixCreateConstructor apply fail because couldnt find a NewExpression from node returned by sourcefile..getDescendantAtPos(positionOrRangeToNumber(positionOrRange) - returned node kind was ${getKindName(originalKind)}`)
      return
    }
    const argTypes = node.getArguments().map(arg => arg.getType().getApparentType().getText())
    const classDeclaration = node.getExpression().getSymbol().getDeclarations()[0]
    if (TypeGuards.isClassDeclaration(classDeclaration)) {
      classDeclaration.addConstructor({
        // docs: 'Autogenerated constructor', //TODO. add some jsdoc to generated constructor
        scope: Scope.Public,
        parameters: argTypes.map(type => ({
          name: `a${type.substring(0, 1).toUpperCase()}${type.substring(1, type.length)}`,
          hasQuestionToken: false,
          type,
          isRestParameter: false,
          // scope: Scope.,
        })
        )
      })
    } else {
      log(`codeFixCreateConstructor apply fail because node is not ClassDeclaration is ${getKindName(classDeclaration.getKind())}`)
    }
  }
};
